// 002 u_c_dbx4_ini
// 17 sep 2007

// -- (C) Felix John COLIBRI 2007
// -- documentation: http://www.felix-colibri.com

(*$r+*)

// -- a lot of loading / unloading the same .INI

unit u_c_dbx4_ini;
  interface
    uses Classes
        , DbxCommon
        , u_c_basic_object
        ;

    const k_default_trace_file_name= 'dbx_trace.txt';

          k_trace_suffix= '_TRACE';
          k_file_trace_suffix= '_TRACE_W_FILE';
          k_pool_suffix= '_POOL';
          k_pool_trace_suffix= '_POOL'+ k_file_trace_suffix;

          k_console_delegation= 'DBXTRACECONNECTION';
          k_console_delgation_line= 'DelegateConnection='+ k_console_delegation;
          k_file_delegation= 'DBXTRACECONNECTION_W_FILE';
          k_file_delgation_line= 'DelegateConnection='+ k_file_delegation;

          k_pool_trace_delegation= 'DBXPOOLTRACECONNECTION_W_FILE';

    type c_dbx4_connections_ini= class(c_basic_object)
                                   m_c_readonly_connection_name_list: tStringList;

                                   Constructor create_dbx4_connections_ini(p_name: String);

                                   function f_connection_name_exists(p_connection_name: String): Boolean;

                                   function f_is_readonly_connection(p_connection_name: String): Boolean;
                                   function f_delete_section(p_connection_name: String): Boolean;

                                   function f_append_new_section(p_connection_name: String;
                                       p_c_connection_parameters: tStrings): Boolean;
                                   function f_update_section(p_connection_name: String;
                                       p_c_connection_parameters: tStrings): Boolean;
                                   function f_update_or_append_section(p_connection_name: String; p_c_connection_parameters: tStrings): Boolean;

                                   procedure append_dbx4_file_trace_connection(p_file_delegation_connection_name: String);
                                   procedure append_dbx4_pool_file_trace_connection(
                                       p_trace_delegation_connection_name, p_pool_trace_delegation_connection_name: String);
                                   procedure write_string(p_section, p_key, p_value: String);

                                   procedure append_pool_delegate_to(p_start_connection_name: String);
                                   procedure append_pool_file_trace_delegate_to(p_start_connection_name: String);

                                   Destructor Destroy; Override;
                                 end; // c_dbx4_connections_ini

    function f_connections_ini_file_name: String;
    function f_drivers_ini_file_name: String;

    procedure refresh_connection_factory;

    procedure get_connection_parameter_list(p_connection_name: String; p_c_strings: tStrings);
    function f_is_valid_parameter_list(p_c_connection_parameters: tStrings): Integer;

    procedure get_connection_name_list(p_c_strings: tStrings);

  implementation
    uses Windows, SysUtils, Registry  // get ini path+ file names
        , IniFiles // read the .INI
        , u_c_display
        ;

    function f_connections_ini_file_name: String;
      var l_c_registry: TRegistry;
      begin
        Result:= TDBXConnectionFile;
        l_c_registry:= TRegistry.Create;
        try
          l_c_registry.RootKey:= HKEY_CURRENT_USER;
          if l_c_registry.OpenKeyReadOnly(TDBXRegistryKey)
            then Result:= l_c_registry.ReadString(TDBXRegistryConnectionValue);
        finally
          l_c_registry.Free;
        end;
      end; // f_connections_ini_file_name

    function f_drivers_ini_file_name: String;
      var l_c_registry: TRegistry;
      begin
        Result:= TDBXDriverFile;
        l_c_registry:= TRegistry.Create;
        try
          l_c_registry.RootKey:= HKEY_CURRENT_USER;
          if l_c_registry.OpenKeyReadOnly(TDBXRegistryKey)
            then Result:= l_c_registry.ReadString(TDBXRegistryDriverValue);
        finally
          l_c_registry.Free;
        end;
      end; // f_drivers_ini_file_name

    procedure refresh_connection_factory;
      begin
        with TDBXConnectionFactory.GetConnectionFactory do
        begin
          Close;
          Open;
       end; // with TDBXConnectionFactory
     end; // refresh_connection_factory

    procedure get_connection_parameter_list(p_connection_name: String; p_c_strings: tStrings);
      var l_c_ini_file: TMemIniFile;
      begin
        l_c_ini_file:= TMemIniFile.Create(f_connections_ini_file_name);
        l_c_ini_file.ReadSectionValues(p_connection_name, p_c_strings);
        l_c_ini_file.Free;
      end; // get_connection_parameter_list

    function f_is_valid_parameter_list(
        p_c_connection_parameters: tStrings): Integer;
        // -- Sanity: check that all memo lines have the "key=value" syntax
        // -- or start with ";" at index 1 (comment)
      var l_key_index: Integer;
      begin
        Result:= -1;

        with p_c_connection_parameters do
          for l_key_index:= 0 to Count- 1 do
            if (Trim(Strings[l_key_index])<> '')
                and
                 (
                   (Pos('=', Strings[l_key_index])<= 0)
                   and not (Strings[l_key_index][1]= ';')
                 )
              then begin
                  Result:= l_key_index;
                  exit;
                end;
      end; // f_is_valid_parameter_list

    procedure get_connection_name_list(p_c_strings: tStrings);
      begin
        // -- since appends, clear tStrings before
        p_c_strings.Clear;
        // -- fetch the singleton and retrieve the names
        TDBXConnectionFactory.GetConnectionFactory.GetConnectionItems(p_c_strings);
      end; // get_connection_name_list

    // -- c_dbx4_connections_ini

    Constructor c_dbx4_connections_ini.create_dbx4_connections_ini(p_name: String);
      begin
        Inherited create_basic_object(p_name);

        m_c_readonly_connection_name_list:= tStringList.Create;

        with m_c_readonly_connection_name_list do
        begin
          CaseSensitive:= False;
          Sorted:= True;

          // -- add the default connections
          Add('DBXPOOLCONNECTION');
          Add('DBXTRACECONNECTION');
          Add('BLACKFISHSQLCONNECTION');
          Add('IBCONNECTION');
          Add('MYSQLCONNECTION');
          Add('ASACONNECTION');
          Add('DB2CONNECTION');
          Add('ORACLECONNECTION');
          Add('INFORMIXCONNECTION');
          Add('MSSQLCONNECTION');
          Add('ASECONNECTION');
        end; // with m_c_readonly_connection_name_list
      end; // create_dbx4_connections_ini

    function c_dbx4_connections_ini.f_connection_name_exists(p_connection_name: String): Boolean;
      var l_c_connection_name_list: tStringList;
      begin
        l_c_connection_name_list:= tStringList.Create;
        get_connection_name_list(l_c_connection_name_list);
        Result:= l_c_connection_name_list.IndexOf(p_connection_name)>= 0;
        l_c_connection_name_list.Free;
      end; // f_connection_name_exists

    function c_dbx4_connections_ini.f_is_readonly_connection(p_connection_name: String): Boolean;
        // -- true if in m_c_readonly_connection_name_list
      begin
        Result:= m_c_readonly_connection_name_list.IndexOf(p_connection_name)>= 0;
      end; // f_is_readonly_connection

    function c_dbx4_connections_ini.f_delete_section(p_connection_name: String): Boolean;
      var l_c_ini_file: TMemIniFile;
      begin
        Result:= not f_is_readonly_connection(p_connection_name);
        if not Result
          then display_bug_stop('readonly');

        if Result
          then begin
              l_c_ini_file:= TMemIniFile.Create(f_connections_ini_file_name);

              l_c_ini_file.EraseSection(p_connection_name);

              l_c_ini_file.UpdateFile;
              l_c_ini_file.Free;
            end;
      end; // f_delete_section

    function c_dbx4_connections_ini.f_append_new_section(p_connection_name: String;
        p_c_connection_parameters: tStrings): Boolean;
      var l_c_dbxconnection_list: tStringList;
          l_key_index: Integer;
          l_the_line: String;
      begin
        display('> f_append_new_section');
        Result:= not f_is_readonly_connection(p_connection_name);
        if not Result
          then Exit;

        l_c_dbxconnection_list:= tStringList.Create;
        with l_c_dbxconnection_list do
        begin
          LoadFromFile(f_connections_ini_file_name);

          // -- add a blank separator line
          Add('');

          // -- add the section
          Add('['+ p_connection_name+ ']');

          // -- copy our memo
          for l_key_index:= 0 to p_c_connection_parameters.Count- 1 do
          begin
            l_the_line:= p_c_connection_parameters.Strings[l_key_index];
            if Trim(l_the_line)<> ''
              then Add(l_the_line);
          end; // for l_key_index

          SaveToFile(f_connections_ini_file_name);

          Free;
        end; // with l_c_dbxconnection_list
        display('< f_append_new_section');
      end; // f_append_new_section

    function c_dbx4_connections_ini.f_update_section(p_connection_name: String;
        p_c_connection_parameters: tStrings): Boolean;
      var l_c_dbxconnection_list: tStringList;
          l_connection_position: Integer;
          l_list_index: Integer;
          l_the_line: String;

          l_key_index: Integer;
      begin
        display('> f_update_section');
        Result:= not f_is_readonly_connection(p_connection_name);
        if not Result
          then Exit;

        l_c_dbxconnection_list:= tStringList.Create;
        with l_c_dbxconnection_list do
        begin
          LoadFromFile(f_connections_ini_file_name);

          // -- find the [my_connection] section
          // -- ?? is the CONNECTIONS.INI case insensitive ?
          l_connection_position:= IndexOf('['+ p_connection_name+ ']');

          l_list_index:= l_connection_position+ 1;
          while l_list_index< Count do
          begin
            l_the_line:= Strings[l_list_index];
            if (Length(l_the_line)> 0) and (l_the_line[1]= '[')
              then Break
              else Delete(l_list_index);
          end; // while l_connection_position

          // -- here on next section or at end of file

          // -- copy our memo
          for l_key_index:= 0 to p_c_connection_parameters.Count- 1 do
          begin
            l_the_line:= p_c_connection_parameters.Strings[l_key_index];
            if Trim(l_the_line)<> ''
              then begin
                  Insert(l_list_index, l_the_line);
                  Inc(l_list_index);
              end;
          end; // for l_key_index

          // -- add a blank line
          if l_list_index< Count
            then Insert(l_list_index, '');

          // -- save to disc
          SaveToFile(f_connections_ini_file_name);
        end; // with l_c_dbxconnection_list
        display('< f_update_section');
      end; // f_update_section

    function c_dbx4_connections_ini.f_update_or_append_section(p_connection_name: String;
        p_c_connection_parameters: tStrings): Boolean;
      var l_c_dbxconnection_list: tStringList;
          l_key_index: Integer;
      begin
        display('> f_update_or_append_section');
        l_c_dbxconnection_list:= tStringList.Create;

        with l_c_dbxconnection_list do
        begin
          LoadFromFile(f_connections_ini_file_name);
          if IndexOf('['+ p_connection_name+ ']')< 0
            then Result:= f_append_new_section(p_connection_name, p_c_connection_parameters)
            else Result:= f_update_section(p_connection_name, p_c_connection_parameters);

          Free;
        end; // with l_c_dbxconnection_list
        display('< f_update_or_append_section');
      end; // f_update_or_append_section

    procedure c_dbx4_connections_ini.append_dbx4_file_trace_connection(p_file_delegation_connection_name: String);
      var l_c_dbx4_file_trace_parameters: tStringList;
      begin
        l_c_dbx4_file_trace_parameters:= tStringList.Create;

        with l_c_dbx4_file_trace_parameters do
        begin
          Add('DriverName=DBXTrace');
          Add('TraceFlags=NONE');
          Add('TraceFile='+ k_default_trace_file_name);
          Add('TraceDriver=False');

          f_append_new_section(p_file_delegation_connection_name, l_c_dbx4_file_trace_parameters);

          Free;
        end; // with l_c_dbx4_file_trace_parameters
      end; // append_dbxtrace_connection

    procedure c_dbx4_connections_ini.append_dbx4_pool_file_trace_connection(
        p_trace_delegation_connection_name, p_pool_trace_delegation_connection_name: String);
      var l_c_dbx4_file_trace_parameters: tStringList;
      begin
        if f_connection_name_exists(p_pool_trace_delegation_connection_name)
          then Exit;

        l_c_dbx4_file_trace_parameters:= tStringList.Create;

        with l_c_dbx4_file_trace_parameters do
        begin
          Add('DriverName=DBXPool');
          Add('MaxConnections=16');
          Add('MinConnections=0');
          Add('ConnectTimeout=0');
          Add('DelegateConnection='+ p_trace_delegation_connection_name);

          f_append_new_section(p_pool_trace_delegation_connection_name, l_c_dbx4_file_trace_parameters);

          Free;
        end; // with l_c_dbx4_file_trace_parameters
      end; // append_dbx4_pool_file_trace_connection

    procedure c_dbx4_connections_ini.write_string(p_section, p_key, p_value: String);
      begin
        with TMemIniFile.Create(f_connections_ini_file_name) do
        begin
          WriteString(p_section, p_key, p_value);
          UpdateFile;
          Free;
        end;
      end; // write_string

    procedure c_dbx4_connections_ini.append_pool_delegate_to(p_start_connection_name: String);
        // -- create a new "p_start_connection_name + _POOL" connection
        // -- with "delegateconnection=DBXPOOLCONNECTION" at the end
      var l_c_connection_name_list: tStringList;
          l_new_connection_name: String;
          l_c_ini_file: TMemIniFile;
          l_c_new_connection_parameters: tStringList;
      begin
        l_new_connection_name:= p_start_connection_name+ '_POOL';
        l_c_connection_name_list:= tStringList.Create;
        get_connection_name_list(l_c_connection_name_list);

        if l_c_connection_name_list.IndexOf(l_new_connection_name)<= 0
          then begin
              l_c_new_connection_parameters:= tStringList.Create;

              l_c_ini_file:= TMemIniFile.Create(f_connections_ini_file_name);
              l_c_ini_file.ReadSectionValues(p_start_connection_name,
                  l_c_new_connection_parameters);
              l_c_ini_file.Free;

              with l_c_new_connection_parameters do
              begin
                Add('delegateconnection=DBXPOOLCONNECTION');
                f_append_new_section(l_new_connection_name,
                    l_c_new_connection_parameters);
                Free;
              end; // with l_c_new_connection_parameters
            end;

        l_c_connection_name_list.Free;
      end; // append_pool_delegate_to

    procedure c_dbx4_connections_ini.append_pool_file_trace_delegate_to(p_start_connection_name: String);
      var l_c_connection_name_list: tStringList;

          l_new_connection_name: String;
          l_c_ini_file: TMemIniFile;
          l_c_new_connection_parameters: tStringList;
      begin
        // -- must have a "base" connction name
        if    ( Pos(k_trace_suffix, p_start_connection_name)> 0)
            or
              ( Pos(k_file_trace_suffix, p_start_connection_name)> 0)
            or
              ( Pos(k_pool_trace_suffix, p_start_connection_name)> 0)
          then Exit;

        l_c_connection_name_list:= tStringList.Create;
        get_connection_name_list(l_c_connection_name_list);

        l_new_connection_name:= p_start_connection_name+ k_pool_suffix+ k_file_trace_suffix;

        if l_c_connection_name_list.IndexOf(l_new_connection_name)<= 0
          then begin
              l_c_new_connection_parameters:= tStringList.Create;

              l_c_ini_file:= TMemIniFile.Create(f_connections_ini_file_name);
              // -- get the "base" connection
              l_c_ini_file.ReadSectionValues(p_start_connection_name,
                  l_c_new_connection_parameters);
              l_c_ini_file.Free;

              with l_c_new_connection_parameters do
              begin
                Add('delegateconnection='+ k_pool_trace_delegation);
                f_append_new_section(l_new_connection_name,
                    l_c_new_connection_parameters);
                Free;
              end; // with l_c_new_connection_parameters
            end;

        l_c_connection_name_list.Free;
      end; // append_pool_file_trace_delegate_to

    Destructor c_dbx4_connections_ini.Destroy;
      begin
        m_c_readonly_connection_name_list.Free;
        Inherited;
      end; // Destroy

    begin // u_c_dbx4_connections_ini
    end. // u_c_dbx4_connections_ini


